using UnityEngine;namespace PluginIdea.DecalSystem{public struct ProjectionSettings{public float vertShift, horzShift, horzKeystone, vertKeystone, projectionRatio, aspectRatio;public ProjectionSettings(float verticalShift, float horizontalShift, float horizontalKeystone, float verticalKeystone, float throwRatio, float imageAspectRatio){vertShift = verticalShift;horzShift = horizontalShift;horzKeystone = horizontalKeystone;vertKeystone = verticalKeystone;projectionRatio = throwRatio;aspectRatio = imageAspectRatio;}}public class DecalProjectionManager{private Texture decalTexture = null;private ProjectionSettings settings;private float decalWidth, decalHeight;private float maxEdgeDistance;private float unitConversion = 1f;private int resolution = 1024;const float projectionDistance = 10f;public Light redChannelLight, greenChannelLight, blueChannelLight;public Texture redDecal, greenDecal, blueDecal;private static Material redChannelMaterial, greenChannelMaterial, blueChannelMaterial;private Vector2 scale, offset;public DecalProjectionManager(ProjectionSettings projSettings, int decalResolution, Light redLight, Light greenLight, Light blueLight, Texture decal = null){resolution = decalResolution;decalTexture = decal;PrepareDecals();redChannelLight = redLight;greenChannelLight = greenLight;blueChannelLight = blueLight;settings = projSettings;InitializeDecalProjection();}void PrepareDecals(){redDecal = new RenderTexture(resolution, resolution, 0, RenderTextureFormat.ARGB32);greenDecal = new RenderTexture(resolution, resolution, 0, RenderTextureFormat.ARGB32);blueDecal = new RenderTexture(resolution, resolution, 0, RenderTextureFormat.ARGB32);redDecal.wrapMode = TextureWrapMode.Clamp;greenDecal.wrapMode = TextureWrapMode.Clamp;blueDecal.wrapMode = TextureWrapMode.Clamp;}void InitializeDecalProjection(){if (redChannelMaterial == null){var shader = RealProjector.dic_BundleShaders["Hidden/DecalRealProjector"];redChannelMaterial = new Material(shader);greenChannelMaterial = new Material(shader);blueChannelMaterial = new Material(shader);redChannelMaterial.SetInt("_TargetChannel", 0);greenChannelMaterial.SetInt("_TargetChannel", 1);blueChannelMaterial.SetInt("_TargetChannel", 2);}if (decalTexture == null){decalTexture = Texture2D.blackTexture;}decalWidth = projectionDistance / settings.projectionRatio;decalHeight = decalWidth / settings.aspectRatio;float shiftHorz = decalWidth * (settings.horzShift / 200f);float shiftVert = decalHeight * (settings.vertShift / 200f);float limitHorz = (decalWidth / 2f) + Mathf.Abs(shiftHorz);float limitVert = (decalHeight / 2f) + Mathf.Abs(shiftVert);maxEdgeDistance = Mathf.Max(limitHorz, limitVert);float lightAngle = Mathf.Atan(maxEdgeDistance / projectionDistance) * 2 * Mathf.Rad2Deg;redChannelLight.spotAngle = greenChannelLight.spotAngle = blueChannelLight.spotAngle = lightAngle;float totalHeight = projectionDistance * Mathf.Tan((lightAngle / 2) * Mathf.Deg2Rad) * 2;float totalWidth = totalHeight;scale = new Vector2(totalWidth / decalWidth, totalHeight / decalHeight);offset = new Vector2(-((totalWidth / 2 + (shiftHorz - decalWidth / 2)) / decalWidth), -((totalHeight / 2 + (shiftVert - decalHeight / 2)) / decalHeight));UpdateDecal();}public void UpdateDecal(){if (decalTexture == null) return;Shader.SetGlobalInt("_Size", resolution);Shader.SetGlobalVector("_PPT", new Vector4(scale.x, scale.y, offset.x, offset.y));Graphics.Blit(decalTexture, redDecal as RenderTexture, redChannelMaterial);Graphics.Blit(decalTexture, greenDecal as RenderTexture, greenChannelMaterial);Graphics.Blit(decalTexture, blueDecal as RenderTexture, blueChannelMaterial);}public void UpdateSettings(ProjectionSettings newSettings){settings = newSettings;InitializeDecalProjection();}public void SetDecalTexture(Texture2D newTexture){decalTexture = newTexture;}}}